#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>

#define LED_ON()    (PORTB |= 1<<PB4)
#define LED_OFF()   (PORTB &= ~(1<<PB4))
#define LED_SWITCH()    (PORTB ^= 1<<PB4)
#define LED_STATUS  (PORTB & (1<<PB4))

#define BUTTON_1 (!(PINB & (1 << PB3)))
#define BUTTON_2 (!(PINB & (1 << PB0)))

const int pulse_up = 0;
const int pulse_down = 1;
const int bright = 2;
const int RAVE = 3;

int main(void)
{
    // Make the LED an output
    DDRB |= 1<<PB4;
        
    // Enable timer
    TCCR0B |= (1 << CS01);
    TIMSK |= (1 << OCIE0A) | (1 << TOIE0);

    // Internal pull-ups on buttons
    PORTB |= (1 << PB3) | (1 << PB0);
    
    int min = 25, max = 75;
    OCR0A = min;
    int mode = pulse_up;
    
    // Turn on global interrupts            
    sei();


    int button1_held = 0;
    int button2_held = 0;
    
    while(1)
    {
    
        int button1 = BUTTON_1;
        int button2 = BUTTON_2;
        
        if (button1 && !button1_held) {
            if (mode != bright) {
                mode = bright;
                cli();
            } else {
                mode = pulse_up;
                OCR0A = min;
                sei();
            }
        }
        
        if (button2 && !button2_held) {
            if (mode != RAVE) {
                mode = RAVE;
                cli();
            } else {
                mode = pulse_up;
                OCR0A = min;
                sei();
            }
        }

        // Debounce
        if (button1) button1_held = 2;
        if (button2) button2_held = 2;
        
        if (button1_held) button1_held--;
        if (button2_held) button2_held--;
            
        if (mode == bright) {
            LED_ON();
        }
        else if (mode == pulse_up) {
            if (++OCR0A == max)
                mode = pulse_down;
            _delay_ms(20);
        } else if (mode == pulse_down) {
            if (--OCR0A == min)
                mode = pulse_up;
            _delay_ms(20);
        } else if (mode == RAVE) {
            LED_SWITCH();
            if (LED_STATUS)
                _delay_ms(25);
            else
                _delay_ms(100);
        }

    }

}

ISR(TIM0_OVF_vect)
{
    LED_ON();
}

ISR(TIM0_COMPA_vect)
{
    LED_OFF();
}